@@ -1,6 +1,6 @@
 package Log::Log4perl::Tiny;
 {
-  $Log::Log4perl::Tiny::VERSION = '1.2.5_01';
+  $Log::Log4perl::Tiny::VERSION = '1.2.5_2';
 }
 
 # ABSTRACT: mimic Log::Log4perl in one single module
@@ -11,6 +11,7 @@ use Carp;
 
 our ($TRACE, $DEBUG, $INFO, $WARN, $ERROR, $FATAL, $OFF, $DEAD);
 my ($_instance, %name_of, %format_for, %id_for);
+my $LOGDIE_MESSAGE_ON_STDERR = 1;
 
 sub import {
    my ($exporter, @list) = @_;
@@ -78,6 +79,9 @@ sub import {
          get_logger()->_set_level_if_first($DEAD);
          $level_set = 1;
       }
+      elsif (lc($item) eq ':no_extra_logdie_message') {
+         $LOGDIE_MESSAGE_ON_STDERR = 0;
+      }
    } ## end for my $item (@list)
 
    if (!$level_set) {
@@ -95,36 +99,22 @@ sub new {
 
    $args{format} = $args{layout} if exists $args{layout};
 
-   my $fh;
-   if (exists $args{file_append}) {
-      open $fh, '>>', $args{file_append}
-        or die "open('$args{file_append}') for appending: $!";
-   }
-   elsif (exists $args{file_create}) {
-      open $fh, '>', $args{file_create}
-        or die "open('$args{file_create}') for creating: $!";
+   my $channels_input = [ fh => \*STDERR ];
+   if (exists $args{channels}) {
+      $channels_input = $args{channels};
    }
    else {
-      my $filename = exists $args{file_insecure} ? $args{file_insecure}
-                   : exists $args{file}          ? $args{file}
-                   :                               undef;
-      if (defined $filename) {
-         open $fh, $filename
-           or die "open('$filename'): $!";
+      for my $key (qw< file_append file_create file_insecure file fh >) {
+         next unless exists $args{$key};
+         $channels_input = [ $key => $args{$key} ];
+         last;
       }
    }
-
-   # autoflush new filehandle and add to args if applicable
-   if (defined $fh) {
-      my $previous = select($fh);
-      $|++;
-      select($previous);
-
-      $args{fh} = $fh;
-   } ## end if (exists $args{file})
+   my $channels = build_channels($channels_input);
+   $channels = $channels->[0] if @$channels == 1; # remove outer shell
 
    my $self = bless {
-      fh    => \*STDERR,
+      fh    => $channels,
       level => $INFO,
    }, $package;
 
@@ -138,6 +128,55 @@ sub new {
    return $self;
 } ## end sub new
 
+sub build_channels {
+   my @pairs = (@_ && ref($_[0])) ? @{$_[0]} : @_;
+   my @channels;
+   while (@pairs) {
+      my ($key, $value) = splice @pairs, 0, 2;
+
+      # some initial validation
+      croak "build_channels(): undefined key in list"
+         unless defined $key;
+      croak "build_channels(): undefined value for key $key"
+         unless defined $value;
+
+      # analyze the key-value pair and set the channel accordingly
+      my ($channel, $set_autoflush);
+      if ($key =~ m{\A(?: fh | sub | code | channel )\z}mxs) {
+         $channel = $value;
+      }
+      elsif ($key eq 'file_append') {
+         open $channel, '>>', $value
+           or croak "open('$value') for appending: $!";
+         $set_autoflush = 1;
+      }
+      elsif ($key eq 'file_create') {
+         open $channel, '>', $value
+           or croak "open('$value') for creating: $!";
+         $set_autoflush = 1;
+      }
+      elsif ($key =~ m{\A file (?: _insecure )? \z}mxs) {
+         open $channel, $value
+           or croak "open('$value'): $!";
+         $set_autoflush = 1;
+      }
+      else {
+         croak "unsupported channel key '$key'";
+      }
+
+      # autoflush new filehandle if applicable
+      if ($set_autoflush) {
+         my $previous = select($channel);
+         $|++;
+         select($previous);
+      } ## end if (exists $args{file})
+
+      # record the channel, on to the next
+      push @channels, $channel;
+   }
+   return \@channels;
+}
+
 sub get_logger { return $_instance ||= __PACKAGE__->new(); }
 sub LOGLEVEL { return get_logger()->level(@_); }
 
@@ -186,7 +225,10 @@ sub log {
      map { $format_for{$_}[1]->(\%data_for); } @{$self->{args}};
 
    my $fh = $self->{fh};
-   ref($fh) eq 'CODE' ? $fh->($message, $self) : print {$fh} $message;
+   for my $channel ((ref($fh) eq 'ARRAY') ? (@$fh) : ($fh)) {
+      (ref($channel) eq 'CODE') ? $channel->($message, $self)
+                                : print {$channel} $message;
+   }
 
    return;
 } ## end sub log
@@ -214,7 +256,7 @@ sub logwarn {
       if substr($_[-1], -1, 1) ne "\n";
 
    # go for it!
-   CORE::warn(@_);
+   CORE::warn(@_) if $LOGDIE_MESSAGE_ON_STDERR;
 } ## end sub logwarn
 
 sub logdie {
@@ -230,7 +272,7 @@ sub logdie {
       if substr($_[-1], -1, 1) ne "\n";
 
    # go for it!
-   CORE::die(@_);
+   CORE::die(@_) if $LOGDIE_MESSAGE_ON_STDERR;
 
    $self->_exit();
 } ## end sub logdie
@@ -243,34 +285,62 @@ sub logexit {
 
 sub logcarp {
    my $self = shift;
-   $self->warn(@_);
    require Carp;
    $Carp::Internal{$_} = 1 for __PACKAGE__;
-   Carp::carp(@_);
+   if ($self->is_warn()) { # avoid unless we're allowed to emit
+      my $message = Carp::shortmess(@_);
+      $self->warn($_) for split m{\n}mxs, $message;
+   }
+   if ($LOGDIE_MESSAGE_ON_STDERR) {
+      local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+      Carp::carp(@_);
+   }
+   return;
 } ## end sub logcarp
 
 sub logcluck {
    my $self = shift;
-   $self->warn(@_);
    require Carp;
    $Carp::Internal{$_} = 1 for __PACKAGE__;
-   Carp::cluck(@_);
+   if ($self->is_warn()) { # avoid unless we're allowed to emit
+      my $message = Carp::longmess(@_);
+      $self->warn($_) for split m{\n}mxs, $message;
+   }
+   if ($LOGDIE_MESSAGE_ON_STDERR) {
+      local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+      Carp::cluck(@_);
+   }
+   return;
 } ## end sub logcluck
 
 sub logcroak {
    my $self = shift;
-   $self->fatal(@_);
    require Carp;
    $Carp::Internal{$_} = 1 for __PACKAGE__;
-   Carp::croak(@_);
+   if ($self->is_fatal()) { # avoid unless we're allowed to emit
+      my $message = Carp::shortmess(@_);
+      $self->fatal($_) for split m{\n}mxs, $message;
+   }
+   if ($LOGDIE_MESSAGE_ON_STDERR) {
+      local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+      Carp::croak(@_);
+   }
+   $self->_exit();
 } ## end sub logcroak
 
 sub logconfess {
    my $self = shift;
-   $self->fatal(@_);
    require Carp;
    $Carp::Internal{$_} = 1 for __PACKAGE__;
-   Carp::confess(@_);
+   if ($self->is_fatal()) { # avoid unless we're allowed to emit
+      my $message = Carp::longmess(@_);
+      $self->fatal($_) for split m{\n}mxs, $message;
+   }
+   if ($LOGDIE_MESSAGE_ON_STDERR) {
+      local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+      Carp::confess(@_);
+   }
+   $self->_exit();
 } ## end sub logconfess
 
 sub level {
@@ -493,7 +563,7 @@ Log::Log4perl::Tiny - mimic Log::Log4perl in one single module
 
 =head1 VERSION
 
-version 1.2.5_01
+version 1.2.5_2
 
 =head1 SYNOPSIS
 
@@ -539,8 +609,17 @@ version 1.2.5_01
    $fh->autoflush();
    $logger->fh($fh);
 
+   # Want to multiplex output to different channels?
+   $logger->fh(
+      build_channels(
+         fh          => \*STDERR,
+         file_create => '/var/log/lastrun.log',
+         file_append => '/var/log/overall.log',
+      )
+   );
+
    # Want to handle the output message by yourself?
-   my @queue; # all log messages will be put here
+   my @queue; # e.g. all log messages will be put here
    $logger->fh(sub { push @queue, $_[0] });
 
 =head1 DESCRIPTION
@@ -744,6 +823,10 @@ you're running setuid. These are extensions added by Log::Log4perl::Tiny
 to cope with this specific case (and also to allow you avoid the 2-args
 C<open()> anyway).
 
+Another Log::Log4perl::Tiny extension added as of version 1.3.0 is
+the key C<channels> where you can pass an array reference with
+channels descriptions (see L</build_channels> for details).
+
 The default is to send logging messages to C<STDERR>;
 
 =item C<< layout >>
@@ -968,6 +1051,75 @@ emit log at C<FATAL> level and then call C<Carp::confess()>;
 
 set the minimum log level for sending a log message to the output;
 
+=item C<< build_channels >>
+
+(Not in L<Log::Log4perl>) (Not imported with C<:easy>)
+
+build multiple channels for emitting logs.
+
+   my $channels = build_channels(@key_value_pairs);  # OR
+   my $channels = build_channels(\@key_value_pairs);
+
+The input is a sequence of key-value pairs, provided either as
+a list or through a reference to an array containing them. They
+are not forced into a hash because the same key can appear
+multiple times to initialize multiple channels.
+
+The key specifies the type of the channel, while the value
+is specific to the key:
+
+=over
+
+=item B<< fh >>
+
+value is a filehandle (or anything that can be passed to the
+C<print> function)
+
+=item B<< sub >>
+
+=item B<< code >>
+
+value is a reference to a subroutine. This will be called with
+two positional parameters: the message (already properly formatted)
+and a reference to the logger message
+
+=item B<channel>
+
+whatever can be passed to keys C<fh> or to C<sub>/C<code> above
+
+=item B<< file >>
+
+=item B<< file_insecure >>
+
+=item B<< file_create >>
+
+=item B<< file_append >>
+
+value is the file where log data should be sent.
+
+The first one is kept for compliance with Log::Log4perl::easy_init's way
+of accepting a file. It eventually results in a two-arguments C<open()>
+call, so that you can quickly set how you want to open the file:
+
+   file => '>>/path/to/appended', # append mode
+   file => '>/path/to/new-file',  # create mode
+
+You should avoid doing this, because it is intrinsically insecure and will
+yield an error message when running setuid:
+
+   Insecure dependency in open while running setuid
+
+C<file_insecure> is an alias to C<file>, so that you can explicitly signal
+to the maintainer that you know what you're doing.
+
+C<file_create> and C<file_append> will use the three-arguments C<open()>
+call and thus they don't trigger the error above when running setuid. As
+the respective names suggest the former creates the file from scratch
+(possibly deleting any previous file with the same path) while the latter
+opens the file in append mode.
+
+=back
+
 =back
 
 =head2 Object-Oriented Interface
@@ -992,6 +1144,11 @@ following keys:
 
 =over
 
+=item B<< channels >>
+
+set a list (through an array reference) of channels. See
+L</build_channels> for additional information.
+
 =item B<< file >>
 
 =item B<< file_insecure >>
@@ -1000,29 +1157,14 @@ following keys:
 
 =item B<< file_append >>
 
-set the file where log data should be sent. This option takes precedence
-over L</fh> below.
+set the file where the log data will be sent.
 
 The first one is kept for compliance with Log::Log4perl::easy_init's way
-of accepting a file. It eventually results in a two-arguments C<open()>
-call, so that you can quickly set how you want to open the file:
+of accepting a file. It eventually results in a two-arguments C<open()>,
+so you might want to take care when running in taint mode.
 
-   file => '>>/path/to/appended', # append mode
-   file => '>/path/to/new-file',  # create mode
-
-You should avoid doing this, because it is intrinsically insecure and will
-yield an error message when running setuid:
-
-   Insecure dependency in open while running setuid
-
-C<file_insecure> is an alias to C<file>, so that you can explicitly signal
-to the maintainer that you know what you're doing.
-
-C<file_create> and C<file_append> will use the three-arguments C<open()>
-call and thus they don't trigger the error above when running setuid. As
-the respective names suggest the former creates the file from scratch
-(possibly deleting any previous file with the same path) while the latter
-opens the file in append mode.
+See also L</build_channels> for additional information. This option takes
+precedence over C<fh> described below.
 
 =item B<< format >>
 
@@ -1143,13 +1285,15 @@ By default the level is set to C<$INFO>.
 
 =item C<< fh >>
 
-get/set the output filehandle.
+get/set the output channel.
 
 As an extention over L<Log::Log4perl>,
-you can also set a reference to a subroutine as a filehandle, in
-which case it will be called with two parameters: the message that
-would be print and a reference to the logger object that is calling
-the sub. For example, if you simply want to collect the log
+you can also pass a reference to a subroutine or to an array.
+
+If you set a reference to a sub,
+it will be called with two parameters: the message
+that would be print and a reference to the logger object that is
+calling the sub. For example, if you simply want to collect the log
 messages without actually outputting them anywhere, you can do this:
 
    my @messages;
@@ -1159,6 +1303,12 @@ messages without actually outputting them anywhere, you can do this:
       return;
    });
 
+If you set a reference to an array, each item inside will be used
+for log output; its elements can be either filehandles or sub
+references, used as described above. This is a handy way to set
+multiple output channels (it might be implemented externally
+through a proper subroutine reference of course).
+
 By default this parameter is set to be equal to C<STDERR>.
 
 =item C<< format >>
